"""
Copyright 2021-2023 MICRORISC s.r.o.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

	http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import json
from logging import Logger
from json.decoder import JSONDecodeError

from Setter.Common.PasswordGenerator import PasswordGenerator
from Setter.Exceptions.IOErrorException import IOErrorException
from Setter.Exceptions.JSONErrorException import JSONErrorException
from Setter.Logging import Logging
from Setter.MosquittoConfig import MosquittoConfig


class TempGwConfig:
	"""
	Temp Gateway Daemon configuration class.

	Methods
	-------
	configure_mqtt(instance_id: str) -> None:
		Configure MQTT messaging component instance.
	"""

	__config_dir: str = '/etc/tempgw/'
	__name: str = 'iTemp Cloud Daemon'
	__logger: Logger = Logging.get_logger()

	@classmethod
	def configure_mqtt(cls, instance_id: str) -> None:
		"""
		Configure MQTT messaging component instance.

		Parameters
		----------
		instance_id: str
			instance identifier

		Returns
		-------
		None

		Exceptions
		----------
		IOErrorException
			raised if configuration cannot be read or written to
		JSONDecodeError
			raised if configuration file has invalid json syntax
		"""
		cls.__logger.info('Configuring %s MQTT messaging', cls.__name)
		try:
			file_name = cls.__config_dir + 'shape__MqttService-IqrfDaemon.json'
			with open(file_name, mode='r', encoding='utf-8') as file:
				mqtt_config = json.load(file)

			username = 'itemp-cloud'
			password = PasswordGenerator.generate()
			MosquittoConfig.create_user(username, password)

			mqtt_config['User'] = username
			mqtt_config['Password'] = password

			with open(file_name, mode='w', encoding='utf-8') as file:
				json.dump(mqtt_config, file, indent=4)

			file_name = cls.__config_dir + 'lgmc__IqarosEdge.json'
			with open(file_name, mode='r', encoding='utf-8') as file:
				mqtt_config = json.load(file)

			mqtt_config['iqrfTopic'] = f'gateway/{instance_id}/iqrf/responses'

			with open(file_name, mode='w', encoding='utf-8') as file:
				json.dump(mqtt_config, file, indent=4)

		except IOError as io_error:
			error_str = Logging.io_error(io_error)
			cls.__logger.error(error_str)
			raise IOErrorException(error_str) from io_error
		except JSONDecodeError as json_error:
			error_str = Logging.json_error(json_error, file_name)
			cls.__logger.error(error_str)
			raise JSONErrorException(error_str) from json_error
